#!/usr/bin/env node

'use strict';

/*
* The 8 stages of yeoman generators
* Not all used here, but documented for quick reference
*
* 1. initializing - Your initialization methods (checking current project state, getting configs, etc)
* 2. prompting - Where you prompt users for options (where you'd call this.prompt())
* 3. configuring - Saving configurations and configure the project (creating .editorconfig files and other metadata files)
* 4. default - If the method name doesn't match a priority, it will be pushed to this group.
* 5. writing - Where you write the generator specific files (routes, controllers, etc)
* 6. conflicts - Where conflicts are handled (used internally)
* 7. install - Where installation are run (npm, bower)
* 8. end - Called last, cleanup, say good bye, etc
*/

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AppGenerator = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _safe = require('colors/safe');

var _safe2 = _interopRequireDefault(_safe);

var _yeomanGenerator = require('yeoman-generator');

var _yeomanGenerator2 = _interopRequireDefault(_yeomanGenerator);

var _shelljs = require('shelljs');

var _shelljs2 = _interopRequireDefault(_shelljs);

var _utilities = require('../utilities');

var Utilities = _interopRequireWildcard(_utilities);

var _ora = require('ora');

var _ora2 = _interopRequireDefault(_ora);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var igniteBase = 'ignite-base';
var lockedReactNativeVersion = '0.39.2';
var lockedIgniteVersion = '1.13.0';

var emptyFolder = function emptyFolder(folder) {
  _shelljs2.default.rm('-rf', folder);
  _shelljs2.default.mkdir(folder);
};

// This step needed as of Ignite 1.7.0 and RN 0.33.0
// This wasn't necessary before, might need to be removed at some point
var cleanAndroid = function cleanAndroid(projectFolder) {
  _shelljs2.default.cd(projectFolder + '/android');
  if (/^win/.test(process.platform)) {
    _shelljs2.default.exec('gradlew.bat clean');
  } else {
    _shelljs2.default.exec('./gradlew clean > /dev/null');
  }
  _shelljs2.default.cd('../..');
};

// This step needed as of Ignite 1.11.0 and RN 0.37.0
// This wasn't necessary before, might need to be removed at some point
var removeJestTests = function removeJestTests(projectFolder) {
  _shelljs2.default.rm('-rf', projectFolder + '/__tests__');
};

/**
 * Doctors the AndroidManifest.xml to put in the stuff we need.
 */
var performInserts = function performInserts(name) {
  // Add permissions for push notifications
  // const pushPermissions = `
  //   <permission
  //       android:name="\${applicationId}.permission.C2D_MESSAGE"
  //       android:protectionLevel="signature" />
  //   <uses-permission android:name="\${applicationId}.permission.C2D_MESSAGE" />
  //   <uses-permission android:name="android.permission.VIBRATE" />
  // `

  var networkPermissions = '\n    <uses-permission android:name="android.permission.ACCESS_NETWORK_STATE"/>\n  ';

  // const appEntries = `
  //     <receiver
  //         android:name="com.google.android.gms.gcm.GcmReceiver"
  //         android:exported="true"
  //         android:permission="com.google.android.c2dm.permission.SEND" >
  //         <intent-filter>
  //             <action android:name="com.google.android.c2dm.intent.RECEIVE" />
  //             <category android:name="\${applicationId}" />
  //         </intent-filter>
  //     </receiver>

  //     <service android:name="com.dieam.reactnativepushnotification.modules.RNPushNotificationRegistrationService"/>
  //     <service
  //         android:name="com.dieam.reactnativepushnotification.modules.RNPushNotificationListenerService"
  //         android:exported="false" >
  //         <intent-filter>
  //             <action android:name="com.google.android.c2dm.intent.RECEIVE" />
  //         </intent-filter>
  //     </service>
  // `

  var enforceServicesVersion = '\nconfigurations.all {\n    resolutionStrategy.eachDependency { DependencyResolveDetails details ->\n        if (details.getRequested().getGroup() == \'com.google.android.gms\') {\n            // If different projects require different versions of\n            // Google Play Services it causes a crash on run.\n            // Fix by overriding version for all projects.\n            details.useVersion(\'9.6.1\')\n        }\n    }\n}\n  ';

  // Push disabled for now:
  // Utilities.insertInFile(`${name}/android/app/src/main/AndroidManifest.xml`, 'SYSTEM_ALERT_WINDOW', pushPermissions)
  // Utilities.insertInFile(`${name}/android/app/src/main/AndroidManifest.xml`, 'android:theme', appEntries)
  Utilities.insertInFile(name + '/android/app/src/main/AndroidManifest.xml', 'SYSTEM_ALERT_WINDOW', networkPermissions);
  Utilities.insertInFile(name + '/android/app/build.gradle', 'task copyDownloadableDepsToLibs', enforceServicesVersion, false);
};

/**
 * Doctors Info.plist to allow API examples to openweather.org
 */
var addAPITransportException = function addAPITransportException(name) {
  var addEntry = 'plutil -replace NSAppTransportSecurity -xml \'<dict> <key>NSExceptionDomains</key> <dict> <key>localhost</key> <dict> <key>NSTemporaryExceptionAllowsInsecureHTTPLoads</key> <true/> </dict> <key>api.openweathermap.org</key> <dict> <key>NSTemporaryExceptionAllowsInsecureHTTPLoads</key> <true/> </dict> </dict> </dict>\' ./' + name + '/ios/' + name + '/Info.plist';
  _shelljs2.default.exec(addEntry);
};

/**
 * A green checkmark
 */
var check = _safe2.default.green('✔︎');

/**
 * A red x.
 */
var xmark = _safe2.default.red('𝗫');

/**
 * Is this command installed on ze computer?
 */
var isCommandInstalled = function isCommandInstalled(command) {
  return !!_shelljs2.default.which(command);
};

/**
 * Behold.  The Yeoman generator to install Ignite.
 *
 * These methods get executed top-to-bottom.  The methods that start with an _ are not automatically run.
 * This is a Yeomanism.
 *
 * Also, a few of these functions like `initializing` and `end` are reserved lifecycle methods.
 * This too is a Yeomanism.
 *
 * Finally, if you want to ensure your task finishes before starting the next one, either return a
 * promise, or call `const done = this.async()` before and `done()` once you're finished your task.
 * And yes.  That too is a Yeomanism.
 */

var AppGenerator = exports.AppGenerator = function (_Generators$Base) {
  _inherits(AppGenerator, _Generators$Base);

  function AppGenerator(args, options) {
    _classCallCheck(this, AppGenerator);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(AppGenerator).call(this, args, options));

    _this.argument('name', { type: String, required: true });

    _this.option('latest', {
      desc: 'Use cutting edge React Native Master',
      type: Boolean,
      defaults: false
    });
    return _this;
  }

  /**
   * Entry point.  Let's set this up.
   */


  _createClass(AppGenerator, [{
    key: 'initializing',
    value: function initializing() {
      // this is a fresh install, so let's always clobber the destination.
      this.conflicter.force = true;
      // prep our spinner
      this.spinner = (0, _ora2.default)('starting');

      this.log('-----------------------------------------------');
      this.log(_safe2.default.red('  (                  )   (                   '));
      this.log(_safe2.default.red('  )\\ )   (        ( /(   )\\ )    *   )       '));
      this.log(_safe2.default.red(' (()/(   )\\ )     )\\()) (()/(  ` )  /(   (   '));
      this.log(_safe2.default.red('  /(_)) (()/(    ((_)\\   /(_))  ( )(_))  )\\  '));
      this.log(_safe2.default.red(' (_))    /(_))_   _((_) (_))   (_(_())  ((_) '));
      this.log(' |_ _|  ' + _safe2.default.red('(_))') + ' __| | \\| | |_ _|  |_   _|  | __|');
      this.log('  | |     | (_ | | .` |  | |     | |    | _| ');
      this.log(' |___|     \\___| |_|\\_| |___|    |_|    |___|');
      this.log('-----------------------------------------------');
      this.log('');
      this.log('An unfair headstart for your React Native apps.');
      this.log(_safe2.default.yellow('https://infinite.red/ignite'));
      this.log('');
      this.log('-----------------------------------------------');

      this.log('Igniting ' + _safe2.default.yellow(this.name) + '\n');
    }

    /**
     * Check for react-native.
     */

  }, {
    key: 'findReactNativeCli',
    value: function findReactNativeCli() {
      var animation = Utilities.startStep('Finding react-native', this);

      if (!isCommandInstalled('react-native')) {
        this._logAndExit(xmark + ' Missing react-native - \'npm install -g react-native-cli\'');
      }

      var rnCli = _shelljs2.default.exec('react-native --version', { silent: true }).stdout;
      // verify 1.x.x or higher (we need react-native link)
      if (!rnCli.match(/react-native-cli:\s[1-9]\d*\.\d+\.\d+/)) {
        this._logAndExit(xmark + ' Must have at least version 1.x - \'npm install -g react-native-cli\'');
      }

      animation.finish();
    }

    /**
     * Check if Android is good
     */

  }, {
    key: 'checkAndroid',
    value: function checkAndroid() {
      var animation = Utilities.startStep('Finding android', this);

      if (isCommandInstalled('android')) {
        var androidVersions = _shelljs2.default.exec('android list', { silent: true }).stdout;
        if (androidVersions.match(/android-23/)) {
          animation.finish();
        } else {
          this.spinner.stop();
          this.log(xmark + ' Missing android SDK 23');
        }
      } else {
        this.spinner.stop();
        this.log(xmark + ' Missing android!');
      }
    }

    /**
     * Check for git.
     */

  }, {
    key: 'findGit',
    value: function findGit() {
      var animation = Utilities.startStep('Finding git', this);

      if (!isCommandInstalled('git')) {
        this._logAndExit(xmark + ' Missing git');
      }

      animation.finish();
    }

    /**
     * Do a quick clean up of the template folder.
     */

  }, {
    key: 'cleanBeforeRunning',
    value: function cleanBeforeRunning() {
      var animation = Utilities.startStep('Getting ready for guests', this);

      emptyFolder(this.sourceRoot());

      animation.finish();
    }

    /**
     * Run React Native init.
     */

  }, {
    key: 'reactNativeInit',
    value: function reactNativeInit() {
      var rnVersion = this.options.latest ? 'facebook/react-native' : lockedReactNativeVersion;
      var animation = Utilities.startStep('Running React Native setup version ' + rnVersion + ' (~ 2 minutes-ish)', this);
      var done = this.async();
      var command = 'react-native';
      // use Master if latest flag was set to true - used for early warning tests
      // package.json will be overwritten with package.json.template -> so follow through is needed there
      // Build base project using RN generator
      var commandOpts = ['init', this.name, '--version', rnVersion];
      this.spawnCommand(command, commandOpts, { stdio: 'ignore' }).on('close', function (retCode) {
        animation.finish(retCode);
        done();
      });
    }

    /**
     * Get the git branch or tag that we should check out.
     */

  }, {
    key: '_getGitBranch',
    value: function _getGitBranch() {
      var tag = this.options['tag'] || lockedIgniteVersion;
      // read the user's choice from the source-branch command line option
      var branch = this.options['branch'];
      // check if the user specified a branch
      var isBranchUndefined = typeof branch === 'undefined' || branch === null || branch === 'master' || branch === '';
      // return branch if its defined
      if (!isBranchUndefined) {
        return branch;
      }
      // otherwise return specified tag, or lockedIgniteVersion
      return tag;
    }

    /**
     * Ensure we have the latest Ignite templates.
     */

  }, {
    key: 'downloadLatestIgnite',
    value: function downloadLatestIgnite() {
      var _this2 = this;

      // the default repo
      var defaultRepo = 'https://github.com/infinitered/ignite.git';
      // the repo the user might have asked for via a command line
      var requestedRepo = this.options['repo'];
      // did the user actually ask for a custom repo?
      var useCustomRepo = typeof requestedRepo !== 'undefined' && requestedRepo !== null && requestedRepo !== '';
      // the right repo to use
      var repo = useCustomRepo ? requestedRepo : defaultRepo;
      var branch = this._getGitBranch();
      // start spinner and message
      var animation = Utilities.startStep('Downloading latest Ignite files from ' + repo + '#' + branch, this);

      var done = this.async();
      var command = 'git';
      var commandOpts = ['clone', '--depth', '1', '--branch', branch, repo, this.sourceRoot()];
      this.spawnCommand(command, commandOpts, { stdio: 'ignore' }).on('close', function (retCode) {
        animation.finish(retCode);
        if (retCode === 0) {
          done();
        } else {
          _this2._logAndExit('Failed to clone ' + repo + ' with branch ' + branch);
        }
      });
    }

    /**
     * Helper to copy a file to the destination.
     */

  }, {
    key: '_cpFile',
    value: function _cpFile(fromFilename, toFilename) {
      var from = this.templatePath(igniteBase + '/' + fromFilename);
      var to = this.destinationPath(this.name + '/' + toFilename);
      this.fs.copyTpl(from, to, { name: this.name, reactNativeVersion: lockedReactNativeVersion, igniteVersion: lockedIgniteVersion });
    }

    /**
     * Helper to copy a template to the destination.
     */

  }, {
    key: '_cpTemplate',
    value: function _cpTemplate(filename) {
      this._cpFile(filename + '.template', filename);
    }

    /**
     * Helper to copy a directory to the destination.
     */

  }, {
    key: '_cpDirectory',
    value: function _cpDirectory(directory) {
      this.directory(this.templatePath(igniteBase + '/' + directory), this.destinationPath(this.name + '/' + directory));
    }
  }, {
    key: 'preRinse',
    value: function preRinse() {
      var animation = Utilities.startStep('Pre-rinse', this);

      // Been having reported issues
      //with Multidex still so extra scrubbing!
      _shelljs2.default.rm('-rf', 'android/app/build');
      _shelljs2.default.rm('-rf', 'node_modules/');

      animation.finish();
    }

    /**
     * Let's ignite all up in hurrr.
     */

  }, {
    key: 'copyExistingStuff',
    value: function copyExistingStuff() {
      var animation = Utilities.startStep('Copying Ignite goodies', this);

      this._cpTemplate('README.md');
      this._cpTemplate('package.json');
      this._cpTemplate('.babelrc');
      this._cpTemplate('.env');
      this._cpFile('.ignite.template', '.ignite');
      this._cpFile('index.js.template', 'index.ios.js');
      this._cpFile('index.js.template', 'index.android.js');
      this._cpFile('index.js.template', 'index.android.js');
      this._cpFile('.editorconfig.template', '.editorconfig');
      this._cpDirectory('git_hooks');
      this._cpDirectory('Tests');
      this._cpDirectory('App');
      this._cpDirectory('fastlane');

      animation.finish();
    }

    /**
     * Let's hand tweak the the android manifest,
     */

  }, {
    key: '_updateAndroid',
    value: function _updateAndroid() {
      var animation = Utilities.startStep('Updating android manifest file', this);

      performInserts(this.name);

      animation.finish();
    }

    /**
     * Let's hand tweak PList to allow our API example to work with Transport Security
     */

  }, {
    key: '_updatePList',
    value: function _updatePList() {
      var animation = Utilities.startStep('Updating PList file', this);

      addAPITransportException(this.name);

      animation.finish();
    }

    /**
     * Let's clean up any temp files.
     */

  }, {
    key: '_cleanAfterRunning',
    value: function _cleanAfterRunning() {
      var animation = Utilities.startStep('Cleaning up after messy guests', this);

      emptyFolder(this.sourceRoot());
      cleanAndroid(this.name);
      removeJestTests(this.name);

      animation.finish();
    }

    /**
     * Log an error and exit gracefully.
     */

  }, {
    key: '_logAndExit',
    value: function _logAndExit(finalMessage) {
      this.spinner.stop();
      this.log(finalMessage);
      process.exit(1);
    }

    /**
     * Installs npm then links (old rnpm style)
     * Also, sadly, we need this install the install() function due to how
     * Yeoman times its template copies.  :(
     */

  }, {
    key: 'install',
    value: function install() {
      var _this3 = this;

      // overwrite RN version if we're testing bleeding edge
      if (this.options.latest) {
        this.log(check + ' Force RN Master');
        Utilities.replaceInFile(this.name + '/package.json', '"react-native":', '    "react-native": "github:facebook/react-native",');
      }

      var animation = Utilities.startStep('Installing Ignite dependencies (~ 1 minute-ish)', this);

      var done = this.async();
      var dir = _shelljs2.default.pwd() + '/' + this.name;
      var command = '';
      var commandOpts = [];

      // Use Yarn if it is installed
      if (!_shelljs2.default.which('yarn')) {
        command = 'npm';
        commandOpts.push('install');
        this.log('\n➟ Package manager:\t[' + check + ']︎ npm  |  [ ]︎ yarn');
      } else {
        command = 'yarn';
        commandOpts.push('install');
        this.log('\n➟ Package manager:\t[ ]︎ npm  |  [' + check + ']︎ yarn');
      }

      // Install node modules
      this.spawnCommand(command, commandOpts, { cwd: dir, stdio: 'ignore' }).on('close', function (retCode) {
        animation.finish(retCode);

        // then run the `react-native link` (old rnpm) command
        var linkAnimation = Utilities.startStep('Linking external libs', _this3);
        _this3.spawnCommand('react-native', ['link'], { cwd: dir, stdio: 'ignore' }).on('close', function (linkReturnCode) {
          linkAnimation.finish(linkReturnCode);

          // Push notifications code, disabled for now
          // Causing issues :(
          // update the android manifest
          _this3._updateAndroid();

          // then update Plist
          _this3._updatePList();
          done();
        });
      });
    }

    /**
     * Hold for applause.
     */

  }, {
    key: 'end',
    value: function end() {
      this._cleanAfterRunning();
      this.spinner.stop();
      this.log('');
      this.log('Time to get cooking! 🍽 ');
      this.log('');
      this.log('To run in iOS:');
      this.log(_safe2.default.yellow('  cd ' + this.name));
      this.log(_safe2.default.yellow('  react-native run-ios'));
      this.log('');
      this.log('To run in Android:');
      this.log(_safe2.default.yellow('  cd ' + this.name));
      this.log(_safe2.default.yellow('  react-native run-android'));
      this.log('');
    }
  }]);

  return AppGenerator;
}(_yeomanGenerator2.default.Base);

module.exports = AppGenerator;