'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createFiles = exports.isInFile = exports.replaceInFile = exports.insertInFile = exports.startStep = exports.getConfig = undefined;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _shelljs = require('shelljs');

var _shelljs2 = _interopRequireDefault(_shelljs);

var _safe = require('colors/safe');

var _safe2 = _interopRequireDefault(_safe);

var _ramda = require('ramda');

var _ramda2 = _interopRequireDefault(_ramda);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A green checkmark
 */
var check = _safe2.default.green('✔︎');

/**
 * A red x.
 */
var xmark = _safe2.default.red('𝗫');

// Get ignite config file
var getConfig = exports.getConfig = function getConfig() {
  try {
    return require(_shelljs2.default.pwd() + '/.ignite');
  } catch (e) {
    return null;
  }
};

var startStep = exports.startStep = function startStep(title, generator) {
  generator.spinner.start();
  generator.spinner.text = title;

  return {
    finish: function finish() {
      var retCode = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

      generator.spinner.stop();
      if (retCode === 0) {
        generator.log(check + ' ' + title);
      } else {
        generator.log(xmark + ' ' + title);
      }
    }
  };
};

// Use this sparingly, as file contents are fickle!
var insertInFile = exports.insertInFile = function insertInFile(theFile, theFind, theInsert) {
  var insertAfter = arguments.length <= 3 || arguments[3] === undefined ? true : arguments[3];

  // read full file - Not a great idea if we ever touch large files
  var data = _fs2.default.readFileSync(theFile, 'utf-8');
  var newContents = '';
  // get the full line of first occurance
  var finder = new RegExp('.*' + theFind + '.*', '');
  var matches = data.match(finder);
  // Quick error check
  if (matches.length === 0) throw new Error('\'' + theFind + '\' was not found in file.');

  // insert contents into file
  if (insertAfter) {
    newContents = data.replace(finder, matches[0] + '\n' + theInsert);
  } else {
    newContents = data.replace(finder, theInsert + '\n' + matches[0]);
  }

  // overwrite file with modified contents
  _fs2.default.writeFileSync(theFile, newContents, 'utf-8');
};

var replaceInFile = exports.replaceInFile = function replaceInFile(theFile, theFind, theReplace) {
  // read full file - Not a great idea if we ever touch large files
  var data = _fs2.default.readFileSync(theFile, 'utf-8');
  // get the full line of first occurance
  var finder = new RegExp('.*' + theFind + '.*', '');
  var matches = data.match(finder);
  // Quick error check
  if (matches.length === 0) throw new Error('\'' + theFind + '\' was not found in file.');

  // replace contents
  var newContents = data.replace(finder, '' + theReplace);

  // overwrite file with modified contents
  _fs2.default.writeFileSync(theFile, newContents, 'utf-8');
};

var isInFile = exports.isInFile = function isInFile(theFile, theFind) {
  // read full file - Not a great idea if we ever touch large files
  var data = _fs2.default.readFileSync(theFile, 'utf-8');
  // get the full line of first occurance
  var finder = new RegExp('.*' + theFind + '.*', '');
  var matches = data.match(finder);
  // Quick error check
  return matches || 0;
};

/* ***********************************************************
* createFiles: Copy template files for generator.
* context: normally just 'this'.
* config: Pass an object that looks something like:
*  { templatePath: 'some-file-name.js',
*      destinationPath: 'folder-name',
*      test: true,
*      style: false }
*************************************************************/

var createFiles = exports.createFiles = function createFiles(context, config) {
  var projectUsesTests = _ramda2.default.path(['options', 'testing'], getConfig());
  var appPath = './App/';
  var testPath = './Tests/';
  var jestPath = './__tests__/';

  var dest = _ramda2.default.prop('destinationPath', config);
  var filename = _ramda2.default.split('.', _ramda2.default.prop('templatePath', config))[0];
  var initFilename = _ramda2.default.toUpper(_ramda2.default.head(filename)) + _ramda2.default.slice(1, Infinity, filename); // ie. Initial Letter Capitalized

  var copyFile = function copyFile(obj) {
    return context.fs.copyTpl(context.templatePath(obj.templatePath), context.destinationPath(obj.destinationPath + context.name + obj.ext), { name: context.name });
  };

  // Create the App versions of the template.
  copyFile(_ramda2.default.merge(config, { 'destinationPath': appPath + dest + '/',
    'ext': initFilename + '.js' }));

  // Followed by the Style, if requested
  if (_ramda2.default.prop('style', config) === true) {
    copyFile(_ramda2.default.merge(config, { 'destinationPath': appPath + dest + '/Styles/',
      'ext': initFilename + 'Style.js',
      'templatePath': filename + '.style.template' }));
  }
  // And the tests!
  if (_ramda2.default.prop('test', config) === true && projectUsesTests !== undefined) {
    if (projectUsesTests === 'ava') {
      copyFile(_ramda2.default.merge(config, { 'destinationPath': testPath + dest + '/',
        'ext': initFilename + 'Test.js',
        'templatePath': filename + '.ava.template' }));
    } else {
      // project uses jest. Mocha support coming in a future release! :)
      copyFile(_ramda2.default.merge(config, { 'destinationPath': jestPath + dest + '/',
        'ext': initFilename + 'Test.js',
        templatePath: filename + '.jest.template' }));
    }
  }
};